import React from 'react';
import classnames from 'classnames';
import debounce from 'lodash/debounce';
import getOr from 'lodash/fp/getOr';
import { parseMetadata, suffixMetadata } from '@highlight-ui/utils-commons';
import times from 'lodash/times';
import mapValues from 'lodash/fp/mapValues';
/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

/* global Reflect, Promise */

var extendStatics = function (d, b) {
  extendStatics = Object.setPrototypeOf || {
    __proto__: []
  } instanceof Array && function (d, b) {
    d.__proto__ = b;
  } || function (d, b) {
    for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p];
  };

  return extendStatics(d, b);
};

function __extends(d, b) {
  if (typeof b !== "function" && b !== null) throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
  extendStatics(d, b);

  function __() {
    this.constructor = d;
  }

  d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function () {
  __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

var styles = {
  "heading-xsmall": "Table-module__ooCFLJ3Z__v4-0-22",
  "lightTheme": "Table-module__ctxHKgJe__v4-0-22",
  "heading-small": "Table-module__5vjNe7JH__v4-0-22",
  "defaultTheme": "Table-module__qTQDEWd7__v4-0-22",
  "heading-medium": "Table-module__2Z8Y-qGx__v4-0-22",
  "heading-large": "Table-module__xAxRLe1a__v4-0-22",
  "heading-xlarge": "Table-module__zxrODHm9__v4-0-22",
  "heading-2xl": "Table-module__biD7wK2f__v4-0-22",
  "body-small": "Table-module__pS1MoePr__v4-0-22",
  "body-base": "Table-module__xAJrWJjR__v4-0-22",
  "table": "Table-module__rfZaqjDQ__v4-0-22",
  "body-large": "Table-module__OHhq-MA7__v4-0-22",
  "body-strong-small": "Table-module__yNqygUJ5__v4-0-22",
  "body-strong-base": "Table-module__-fQiU1lb__v4-0-22",
  "body-strong-large": "Table-module__zUmx6zSK__v4-0-22",
  "page-heading": "Table-module__qGXFmWYG__v4-0-22",
  "button": "Table-module__sHQ53oax__v4-0-22",
  "form-label": "Table-module__DV2M-1Na__v4-0-22",
  "caption": "Table-module__R6Ka1Ohk__v4-0-22",
  "text-link-default": "Table-module__EXpeEROZ__v4-0-22",
  "text-link-hover": "Table-module__3jQ1zggY__v4-0-22",
  "table-data-regular": "Table-module__AsU9OQzS__v4-0-22",
  "table-data-bold": "Table-module__MKyvYyRm__v4-0-22",
  "fixedColumnCount1": "Table-module__p6dcXLn-__v4-0-22",
  "fixedColumnCount2": "Table-module__M-byzDQ5__v4-0-22",
  "fixedColumnCount3": "Table-module__wJz7e22E__v4-0-22",
  "fixedColumnCount4": "Table-module__x23Tz3CJ__v4-0-22",
  "fixedColumnCount5": "Table-module__qn3f-QLZ__v4-0-22",
  "fixedHeader": "Table-module__RdeZIv3v__v4-0-22"
};
/* eslint-disable implicit-arrow-linebreak */

var getTableColumnWidth = function (tableElement, index) {
  var tableColumnElement = tableElement.querySelector("th:nth-child(" + (index + 1) + ")");
  return tableColumnElement ? tableColumnElement.getBoundingClientRect().width : 0;
};

var getTableColumnLeftOffset = function (columnWidths, index) {
  return columnWidths.slice(0, index).reduce(function (current, sum) {
    return sum + current;
  }, 0);
};

var getTableCells = function (tableElement, index) {
  return Array.from(tableElement.querySelectorAll("th:nth-child(" + (index + 1) + "), td:nth-child(" + (index + 1) + ")"));
};

var stackColumnsToTheLeft = function (tableElement, columnCount) {
  var columnWidths = times(columnCount).map(function (index) {
    return getTableColumnWidth(tableElement, index);
  });
  var columnOffsets = columnWidths.map(function (_, index) {
    return getTableColumnLeftOffset(columnWidths, index);
  });
  columnOffsets.forEach(function (offset, index) {
    Array.from(getTableCells(tableElement, index)).forEach(function (cell) {
      // eslint-disable-next-line no-param-reassign
      cell.style.left = offset + "px";
    });
  });
};

var renderTh = function (props) {
  var column = props.column,
      metadata = props.metadata;
  return /*#__PURE__*/React.createElement("th", __assign({
    key: column.key
  }, parseMetadata(mapValues(function (x) {
    return x + "-headCell";
  }, metadata))), column.label);
};

var renderTd = function (props) {
  var column = props.column,
      content = props.content,
      metadata = props.metadata;
  return /*#__PURE__*/React.createElement("td", __assign({
    key: column.key
  }, parseMetadata(mapValues(function (x) {
    return x + "-cell";
  }, metadata))), content);
};

var renderTr = function (props) {
  var children = props.children,
      rowIndex = props.rowIndex,
      metadata = props.metadata,
      rowType = props.rowType;
  return /*#__PURE__*/React.createElement("tr", __assign({
    key: rowIndex,
    className: rowType
  }, parseMetadata(mapValues(function (x) {
    return x + "-row";
  }, metadata))), children);
};

var Table =
/** @class */
function (_super) {
  __extends(Table, _super);

  function Table(props) {
    var _this = _super.call(this, props) || this;

    _this.componentDidMount = function () {
      window.addEventListener('resize', _this.debouncedStackFixedColumns);

      _this.stackFixedColumns();
    };

    _this.componentDidUpdate = function () {
      return _this.debouncedStackFixedColumns();
    };

    _this.componentWillUnmount = function () {
      window.removeEventListener('resize', _this.debouncedStackFixedColumns);
    };

    _this.stackFixedColumns = function () {
      var fixedColumnCount = _this.props.fixedColumnCount;
      var tableElement = _this.tableRef.current;
      if (!fixedColumnCount || tableElement == null) return;
      stackColumnsToTheLeft(tableElement, fixedColumnCount);
    };

    _this.tableRef = /*#__PURE__*/React.createRef();
    _this.debouncedStackFixedColumns = debounce(_this.stackFixedColumns, 200);
    return _this;
  }

  Table.prototype.render = function () {
    var _a = this.props,
        className = _a.className,
        _b = _a.columns,
        columns = _b === void 0 ? [] : _b,
        _c = _a.data,
        data = _c === void 0 ? [] : _c,
        _d = _a.fixedColumnCount,
        fixedColumnCount = _d === void 0 ? 0 : _d,
        fixedHeader = _a.fixedHeader,
        _e = _a.renderTd,
        renderTd$1 = _e === void 0 ? renderTd : _e,
        _f = _a.renderTh,
        renderTh$1 = _f === void 0 ? renderTh : _f,
        _g = _a.renderTr,
        renderTr$1 = _g === void 0 ? renderTr : _g,
        metadata = _a.metadata,
        _h = _a.theme,
        theme = _h === void 0 ? 'default' : _h;
    var innerMetadata = suffixMetadata(metadata);
    return /*#__PURE__*/React.createElement("table", __assign({
      className: classnames(className, fixedHeader ? styles.fixedHeader : null, styles.table, styles["fixedColumnCount" + fixedColumnCount], theme ? styles[theme + "Theme"] : styles.defaultTheme),
      ref: this.tableRef
    }, parseMetadata(metadata)), /*#__PURE__*/React.createElement("thead", __assign({}, parseMetadata(innerMetadata('head'))), /*#__PURE__*/React.createElement("tr", __assign({}, parseMetadata(innerMetadata('headRow'))), columns.map(function (column) {
      return __assign({
        renderTh: renderTh$1
      }, column);
    }).map(function (column, columnIndex) {
      return column.renderTh({
        column: column,
        columnIndex: columnIndex,
        metadata: metadata
      });
    }))), /*#__PURE__*/React.createElement("tbody", __assign({}, parseMetadata(innerMetadata('body'))), data.map(function (dataRow, rowIndex) {
      return renderTr$1({
        children: columns.map(function (column) {
          return __assign({
            renderTd: renderTd$1
          }, column);
        }).map(function (column, columnIndex) {
          return column.renderTd({
            column: column,
            columnIndex: columnIndex,
            content: getOr(undefined, column.key || '', dataRow),
            dataRow: dataRow,
            rowIndex: rowIndex,
            metadata: metadata
          });
        }),
        dataRow: dataRow,
        rowIndex: rowIndex,
        metadata: metadata
      });
    })));
  };

  return Table;
}(React.Component);

export { Table };
//# sourceMappingURL=index.js.map
